@@ -27,6 +27,72 @@ def twos(val,bits):
         val = val - (1 << bits)
     return val
 
+def simd_val_comb(xlen, bit_width, signed=True):
+    if type(bit_width)==tuple:
+        bit_width1, bit_width2 = bit_width
+    else:
+        bit_width1, bit_width2 = bit_width, bit_width
+
+    fmt = {8: 'b', 16: 'h', 32: 'w', 64: 'd'}
+    sz1 = fmt[bit_width1]
+    sz2 = fmt[bit_width2]
+    var_num = xlen//bit_width1
+    coverpoints = []
+    for i in range(var_num):
+        var1 = f'rs1_{sz1}{i}_val'
+        var2 = f'rs2_{sz2}{i}_val'
+        if (signed):
+            coverpoints += [(f'{var1} > 0 and {var2} > 0','simd_val_comb')]
+            coverpoints += [(f'{var1} > 0 and {var2} < 0','simd_val_comb')]
+            coverpoints += [(f'{var1} < 0 and {var2} < 0','simd_val_comb')]
+            coverpoints += [(f'{var1} < 0 and {var2} > 0','simd_val_comb')]
+            coverpoints += [(f'{var1} == {var2}','simd_val_comb')]
+            coverpoints += [(f'{var1} != {var2}','simd_val_comb')]
+        else:
+            coverpoints += [(f'{var1} == {var2} and {var1} > 0 and {var2} > 0','simd_val_comb')]
+            coverpoints += [(f'{var1} != {var2} and {var1} > 0 and {var2} > 0','simd_val_comb')]
+
+    return coverpoints
+
+def simd_base_val(rs, xlen, _bit_width, signed=True):
+    fmt = {8: 'b', 16: 'h', 32: 'w', 64: 'd'}
+
+    if type(_bit_width)==tuple:
+        if (rs=="rs1"):
+            bit_width, not_used = _bit_width
+        else:
+            not_used, bit_width = _bit_width
+    else:
+        bit_width, not_used = _bit_width, _bit_width
+
+    sz = fmt[bit_width]
+    var_num = xlen//bit_width
+    sign_val = [(-2**(bit_width-1)), -1, 0, 1, int((2**(bit_width-1)-1))]
+    usign_val = [0, 1, 2**bit_width-2, 2**bit_width-1]
+    coverpoints = []
+    for i in range(var_num):
+        var = f'{rs}_{sz}{i}_val'
+        if (signed):
+            for val in sign_val:
+                coverpoints += [(f'{var} == {val}', 'signed_min_max_middle')]
+            coverpoints += walking_ones(var, bit_width, True)
+            coverpoints += walking_zeros(var, bit_width, True)
+            coverpoints += alternate(var, bit_width, True)
+        else:
+            for val in usign_val:
+                coverpoints += [(f'{var} == {val}','unsigned_min_max_middle')]
+            coverpoints += walking_ones(var, bit_width, False)
+            coverpoints += walking_zeros(var, bit_width, False)
+            coverpoints += alternate(var, bit_width, False)
+    return coverpoints
+
+def simd_imm_val(imm, bit_width):
+    usign_val = (2**(bit_width))
+    coverpoints = []
+    for i in range(usign_val):
+        coverpoints += [(f'{imm} == {i}','simd_imm_val')]
+    return coverpoints
+
 def sp_vals(bit_width,signed):
     if signed:
         conv_func = lambda x: twos(x,bit_width)
@@ -36,7 +102,6 @@ def sp_vals(bit_width,signed):
         sqrt_min = 0
         sqrt_max = int(sqrt((2**bit_width)-1))
         conv_func = lambda x: (int(x,16) if '0x' in x else int(x,2)) if isinstance(x,str) else x
-
     dataset = [3, "0x"+"".join(["5"]*int(bit_width/4)), "0x"+"".join(["a"]*int(bit_width/4)), 5, "0x"+"".join(["3"]*int(bit_width/4)), "0x"+"".join(["6"]*int(bit_width/4))]
     dataset = list(map(conv_func,dataset)) + [int(sqrt(abs(conv_func("0x8"+"".join(["0"]*int((bit_width/4)-1)))))*(-1 if signed else 1))] + [sqrt_min,sqrt_max]
     return dataset + [x - 1 if x>0 else 0 for x in dataset] + [x+1 for x in dataset]
@@ -154,9 +219,11 @@ def byte_count(xlen, variables=['rs1_val','rs2_val','imm_val'], overlap = "N"):
     hex_str = ""
     i=0
     cvpt = ""
-
-    while(i<=256):
-    	hex_str = "{:02x}".format(i) + hex_str
+    max = 255
+    if overlap == "Y":
+    	max += xlen/16
+    while(i<=max):
+    	hex_str = "{:02x}".format(i % 256) + hex_str
     	if((len(hex_str)/2)%(xlen/8) == 0):
     		rs2.append('0x'+hex_str)
     		hex_str = ""
@@ -185,7 +252,7 @@ def byte_count(xlen, variables=['rs1_val','rs2_val','imm_val'], overlap = "N"):
     			else:
     				coverpoints.append(cvpt + ' #nosat')
     			cvpt = ""
-    	elif variables[1] == "rcon":
+    	elif variables[1] == "imm_val":
     		for i in range(len(rs2)):
     			coverpoints.append(variables[0] +' == '+ rs2[i] +' and '+ variables[1] +' == 0xA' + ' #nosat')
     return [(coverpoint,"Byte Count") for coverpoint in coverpoints]
@@ -434,6 +501,7 @@ def expand_cgf(cgf_files, xlen):
                     if 'abstract_comb' in node:
                         temp = node['abstract_comb']
                         del node['abstract_comb']
+
                         for coverpoints, coverage in temp.items():
                             i = 0
                             try:
